@@ -67,7 +67,6 @@ int dplaces_nr = -1;
 int group_nr = 0;	/* Nb of device groups */
 int cpu_nr = 0;		/* Nb of processors on the machine */
 int flags = 0;		/* Flag for common options and system state */
-unsigned int dm_major;	/* Device-mapper major number */
 
 long interval = 0;
 char timestamp[TIMESTAMP_LEN];
@@ -92,14 +91,14 @@ void usage(char *progname)
 	fprintf(stderr, _("Options are:\n"
 			  "[ -c ] [ -d ] [ -h ] [ -k | -m ] [ -N ] [ -s ] [ -t ] [ -V ] [ -x ] [ -y ] [ -z ]\n"
 			  "[ { -f | +f } <directory> ] [ -j { ID | LABEL | PATH | UUID | ... } ]\n"
-			  "[ --dec={ 0 | 1 | 2 } ] [ --human ] [ -o JSON ]\n"
+			  "[ --dec={ 0 | 1 | 2 } ] [ --human ] [ --pretty ] [ -o JSON ]\n"
 			  "[ [ -H ] -g <group_name> ] [ -p [ <device> [,...] | ALL ] ]\n"
 			  "[ <device> [...] | ALL ] [ --debuginfo ]\n"));
 #else
 	fprintf(stderr, _("Options are:\n"
 			  "[ -c ] [ -d ] [ -h ] [ -k | -m ] [ -N ] [ -s ] [ -t ] [ -V ] [ -x ] [ -y ] [ -z ]\n"
 			  "[ { -f | +f } <directory> ] [ -j { ID | LABEL | PATH | UUID | ... } ]\n"
-			  "[ --dec={ 0 | 1 | 2 } ] [ --human ] [ -o JSON ]\n"
+			  "[ --dec={ 0 | 1 | 2 } ] [ --human ] [ --pretty ] [ -o JSON ]\n"
 			  "[ [ -H ] -g <group_name> ] [ -p [ <device> [,...] | ALL ] ]\n"
 			  "[ <device> [...] | ALL ]\n"));
 #endif
@@ -318,8 +317,8 @@ int get_major_minor_nr(char filename[], int *major, int *minor)
 	if (__stat(dfile, &statbuf) < 0)
 		return -1;
 
-	*major = major(statbuf.st_rdev);
-	*minor = minor(statbuf.st_rdev);
+	*major = __major(statbuf.st_rdev);
+	*minor = __minor(statbuf.st_rdev);
 
 	return 0;
 }
@@ -1069,7 +1068,7 @@ void write_disk_stat_header(int *fctr, int *tab, int hpart)
 		return;
 	}
 
-	if (!DISPLAY_HUMAN_READ(flags)) {
+	if (!DISPLAY_PRETTY(flags)) {
 		printf("Device       ");
 	}
 	if (DISPLAY_EXTENDED(flags)) {
@@ -1107,7 +1106,7 @@ void write_disk_stat_header(int *fctr, int *tab, int hpart)
 			       spc, units, spc, units, spc, units, spc, units, spc, units, spc, units);
 		}
 	}
-	if (DISPLAY_HUMAN_READ(flags)) {
+	if (DISPLAY_PRETTY(flags)) {
 		printf(" Device");
 	}
 	printf("\n");
@@ -1143,7 +1142,7 @@ void write_plain_ext_stat(unsigned long long itv, int fctr, int hpart,
 	if (d->dev_tp == T_GROUP)
 		return;
 
-	if (!DISPLAY_HUMAN_READ(flags)) {
+	if (!DISPLAY_PRETTY(flags)) {
 		cprintf_in(IS_STR, "%-13s", devname, 0);
 	}
 
@@ -1280,7 +1279,7 @@ void write_plain_ext_stat(unsigned long long itv, int fctr, int hpart,
 		}
 	}
 
-	if (DISPLAY_HUMAN_READ(flags)) {
+	if (DISPLAY_PRETTY(flags)) {
 		cprintf_in(IS_STR, " %s", devname, 0);
 	}
 	printf("\n");
@@ -1548,7 +1547,7 @@ void write_plain_basic_stat(unsigned long long itv, int fctr,
 {
 	double rsectors, wsectors, dsectors;
 
-	if (!DISPLAY_HUMAN_READ(flags)) {
+	if (!DISPLAY_PRETTY(flags)) {
 		cprintf_in(IS_STR, "%-13s", devname, 0);
 	}
 
@@ -1592,7 +1591,7 @@ void write_plain_basic_stat(unsigned long long itv, int fctr,
 						: (unsigned long long) dc_sec / fctr);
 	}
 
-	if (DISPLAY_HUMAN_READ(flags)) {
+	if (DISPLAY_PRETTY(flags)) {
 		cprintf_in(IS_STR, " %s", devname, 0);
 	}
 	printf("\n");
@@ -1709,7 +1708,7 @@ void write_stats(int curr, struct tm *rectime, int skip)
 	int h, hl = 0, hh = 0, fctr = 1, tab = 4, next = FALSE;
 	unsigned long long itv;
 	struct io_device *d, *dtmp, *g = NULL, *dnext = NULL;
-	char *dev_name, *pdname, *bang, dname[MAX_NAME_LEN];
+	char *dev_name;
 
 	/* Test stdout */
 	TEST_STDOUT(STDOUT_FILENO);
@@ -1748,7 +1747,7 @@ void write_stats(int curr, struct tm *rectime, int skip)
 
 		memset(&iozero, 0, sizeof(struct io_stats));
 
-		if (DISPLAY_HUMAN_READ(flags) &&
+		if (DISPLAY_PRETTY(flags) &&
 		    DISPLAY_EXTENDED(flags) &&
 		    !DISPLAY_SHORT_OUTPUT(flags) &&
 		    !DISPLAY_JSON_OUTPUT(flags)) {
@@ -1835,51 +1834,10 @@ void write_stats(int curr, struct tm *rectime, int skip)
 					    ioj = &iozero;
 				}
 
-				/* Get device name to print */
-				dev_name = NULL;
-
-				if ((DISPLAY_DEVMAP_NAME(flags)) && (d->major == dm_major)) {
-					/*
-					 * If the device is a device mapper device, try to get its
-					 * assigned name of its logical device.
-					 */
-					 dev_name = transform_devmapname(d->major, d->minor);
-				}
-
-				if (!dev_name && ((dev_name = ioc_name(d->major, d->minor)) != NULL)) {
-					if (!strcmp(d->name, dev_name) || !strcmp(dev_name, K_NODEV)) {
-						/*
-						 * Ignore name given by sysstat.ioconf if it's the same
-						 * as current one or if it's "nodev".
-						 * NB: Using names generated from sysstat.ioconf data
-						 * works around known issues with EMC PowerPath.
-						 */
-						dev_name = NULL;
-					}
-				}
-
-				if (DISPLAY_PERSIST_NAME_I(flags)) {
-					pdname = get_persistent_name_from_pretty(dev_name ? dev_name : d->name);
-					if (pdname) {
-						dev_name = pdname;
-					}
-				}
-
-				if (!dev_name) {
-					dev_name = d->name;
-				}
-				strncpy(dname, dev_name, sizeof(dname));
-				dname[sizeof(dname) - 1] = '\0';
-
-				while ((bang = strchr(dname, '!'))) {
-					/*
-					 * Some devices may have had a slash replaced with
-					 * a bang character (eg. cciss!c0d0...)
-					 * Restore their original names.
-					 */
-					*bang = '/';
-				}
-
+				dev_name = get_device_name(d->major, d->minor, NULL, 0,
+							   DISPLAY_DEVMAP_NAME(flags),
+							   DISPLAY_PERSIST_NAME_I(flags),
+							   FALSE, d->name);
 #ifdef DEBUG
 				if (DISPLAY_DEBUG(flags)) {
 					/* Debug output */
@@ -1892,7 +1850,7 @@ void write_stats(int curr, struct tm *rectime, int skip)
 						"fl_ios=%lu fl_ticks=%u "
 						"ios_pgr=%u tot_ticks=%u "
 						"rq_ticks=%u }\n",
-						dname,
+						dev_name,
 						itv,
 						fctr,
 						ioi->rd_sectors,
@@ -1922,10 +1880,10 @@ void write_stats(int curr, struct tm *rectime, int skip)
 					next = TRUE;
 
 					if (DISPLAY_EXTENDED(flags)) {
-						write_ext_stat(itv, fctr, h, d, ioi, ioj, tab, dname);
+						write_ext_stat(itv, fctr, h, d, ioi, ioj, tab, dev_name);
 					}
 					else {
-						write_basic_stat(itv, fctr, d, ioi, ioj, tab, dname);
+						write_basic_stat(itv, fctr, d, ioi, ioj, tab, dev_name);
 					}
 				}
 			}
@@ -2120,6 +2078,12 @@ int main(int argc, char **argv)
 			opt++;
 		}
 
+		else if (!strcmp(argv[opt], "--pretty")) {
+			/* Display an easy-to-read CIFS report */
+			flags |= I_D_PRETTY;
+			opt++;
+		}
+
 #ifdef TEST
 		else if (!strncmp(argv[opt], "--getenv", 8)) {
 			__env = TRUE;
@@ -2165,11 +2129,8 @@ int main(int argc, char **argv)
 				fprintf(stderr, _("Invalid type of persistent device name\n"));
 				exit(1);
 			}
-			/*
-			 * Persistent names are usually long: Display
-			 * them as human readable by default.
-			 */
-			flags |= I_D_PERSIST_NAME + I_D_HUMAN_READ;
+			/* Persistent names are usually long: Pretty display them */
+			flags |= I_D_PERSIST_NAME + I_D_PRETTY;
 			opt++;
 		}
 
@@ -2214,11 +2175,8 @@ int main(int argc, char **argv)
 					break;
 
 				case 'h':
-					/*
-					 * Display device utilization report
-					 * in a human readable format. Also imply --human.
-					 */
-					flags |= I_D_HUMAN_READ + I_D_UNIT;
+					/* Option -h is equivalent to --pretty --human */
+					flags |= I_D_PRETTY + I_D_UNIT;
 					break;
 
 				case 'k':
@@ -2361,10 +2319,6 @@ int main(int argc, char **argv)
 	/* Select disk output unit (kB/s or blocks/s) */
 	set_disk_output_unit();
 
-	if (DISPLAY_DEVMAP_NAME(flags)) {
-		dm_major = get_devmap_major();
-	}
-
 	if (DISPLAY_JSON_OUTPUT(flags)) {
 		/* Use a decimal point to make JSON code compliant with RFC7159 */
 		setlocale(LC_NUMERIC, "C");
