@@ -1,6 +1,6 @@
 /*
  * rd_stats.c: Read system statistics
- * (C) 1999-2019 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -41,6 +41,14 @@
 #define _(string) (string)
 #endif
 
+/* Generic PSI structure */
+struct stats_psi {
+	unsigned long long total;
+	unsigned long	   avg10;
+	unsigned long	   avg60;
+	unsigned long	   avg300;
+};
+
 /*
  ***************************************************************************
  * Read CPU statistics.
@@ -745,7 +753,7 @@ __nr_t read_diskstats_io(struct stats_io *st_io)
 			   &wr_ios, &wr_sec,
 			   &dc_ios, &dc_sec) >= 7) {
 
-			if (is_device(dev_name, IGNORE_VIRTUAL_DEVICES)) {
+			if (is_device(SLASH_SYS, dev_name, IGNORE_VIRTUAL_DEVICES)) {
 				/*
 				 * OK: It's a (real) device and not a partition.
 				 * Note: Structure should have been initialized first!
@@ -820,7 +828,7 @@ __nr_t read_diskstats_disk(struct stats_disk *st_disk, __nr_t nr_alloc,
 			if (!rd_ios && !wr_ios && !dc_ios)
 				/* Unused device: Ignore it */
 				continue;
-			if (read_part || is_device(dev_name, ACCEPT_VIRTUAL_DEVICES)) {
+			if (read_part || is_device(SLASH_SYS, dev_name, ACCEPT_VIRTUAL_DEVICES)) {
 
 				if (dsk_read + 1 > nr_alloc) {
 					dsk_read = -1;
@@ -1040,8 +1048,8 @@ __nr_t read_net_dev(struct stats_net_dev *st_net_dev, __nr_t nr_alloc)
 			}
 
 			st_net_dev_i = st_net_dev + dev_read++;
-			strncpy(iface, line, MINIMUM(pos, MAX_IFACE_LEN - 1));
-			iface[MINIMUM(pos, MAX_IFACE_LEN - 1)] = '\0';
+			strncpy(iface, line, MINIMUM(pos, sizeof(iface) - 1));
+			iface[MINIMUM(pos, sizeof(iface) - 1)] = '\0';
 			sscanf(iface, "%s", st_net_dev_i->interface); /* Skip heading spaces */
 			sscanf(line + pos + 1, "%llu %llu %*u %*u %*u %*u %llu %llu %llu %llu "
 			       "%*u %*u %*u %*u %*u %llu",
@@ -1163,8 +1171,8 @@ __nr_t read_net_edev(struct stats_net_edev *st_net_edev, __nr_t nr_alloc)
 			}
 
 			st_net_edev_i = st_net_edev + dev_read++;
-			strncpy(iface, line, MINIMUM(pos, MAX_IFACE_LEN - 1));
-			iface[MINIMUM(pos, MAX_IFACE_LEN - 1)] = '\0';
+			strncpy(iface, line, MINIMUM(pos, sizeof(iface) - 1));
+			iface[MINIMUM(pos, sizeof(iface) - 1)] = '\0';
 			sscanf(iface, "%s", st_net_edev_i->interface); /* Skip heading spaces */
 			sscanf(line + pos + 1, "%*u %*u %llu %llu %llu %llu %*u %*u %*u %*u "
 			       "%llu %llu %llu %llu %llu",
@@ -2495,7 +2503,7 @@ __nr_t read_filesystem(struct stats_filesystem *st_filesystem, __nr_t nr_alloc)
 {
 	FILE *fp;
 	char line[512], fs_name[MAX_FS_LEN], mountp[256], type[128];
-	int skip = 0, skip_next = 0;
+	int skip = 0, skip_next = 0, fs;
 	char *pos = 0, *pos2 = 0;
 	__nr_t fs_read = 0;
 	struct stats_filesystem *st_filesystem_i;
@@ -2538,12 +2546,13 @@ __nr_t read_filesystem(struct stats_filesystem *st_filesystem, __nr_t nr_alloc)
 			sscanf(line, "%127s", fs_name);
 			/*
 			 * And now read the corresponding mount point.
-			 * Read fs name and mount point in two distinct operations.
+			 * Read fs name and mount point in two distinct operations,
+			 * using '@pos + 1' position value for the mount point.
 			 * Indeed, if fs name length is greater than 127 chars,
-			 * previous scanf() will read only the first 127 chars, and
-			 * mount point name will be read using the remaining chars
-			 * from the fs name. This will result in a bogus name
-			 * and following statvfs() function will always fail.
+			 * previous scanf() would read only the first 127 chars, and
+			 * mount point name would be read using the remaining chars
+			 * from the fs name. This would result in a bogus name
+			 * and following statvfs() function would always fail.
 			 */
 			sscanf(pos + 1, "%255s", mountp);
 
@@ -2557,6 +2566,18 @@ __nr_t read_filesystem(struct stats_filesystem *st_filesystem, __nr_t nr_alloc)
 			if ((__statvfs(mountp, &buf) < 0) || (!buf.f_blocks))
 				continue;
 
+			/* Check if it's a duplicate entry */
+			fs = fs_read - 1;
+			while (fs >= 0) {
+				st_filesystem_i = st_filesystem + fs;
+				if (!strcmp(st_filesystem_i->fs_name, fs_name))
+					break;
+				fs--;
+			}
+			if (fs >= 0)
+				/* Duplicate entry found! Ignore current entry */
+				continue;
+
 			if (fs_read + 1 > nr_alloc) {
 				fs_read = -1;
 				break;
@@ -2568,10 +2589,10 @@ __nr_t read_filesystem(struct stats_filesystem *st_filesystem, __nr_t nr_alloc)
 			st_filesystem_i->f_bavail = (unsigned long long) buf.f_bavail * (unsigned long long) buf.f_frsize;
 			st_filesystem_i->f_files  = (unsigned long long) buf.f_files;
 			st_filesystem_i->f_ffree  = (unsigned long long) buf.f_ffree;
-			strncpy(st_filesystem_i->fs_name, fs_name, MAX_FS_LEN);
-			st_filesystem_i->fs_name[MAX_FS_LEN - 1] = '\0';
-			strncpy(st_filesystem_i->mountp, mountp, MAX_FS_LEN);
-			st_filesystem_i->mountp[MAX_FS_LEN - 1] = '\0';
+			strncpy(st_filesystem_i->fs_name, fs_name, sizeof(st_filesystem_i->fs_name));
+			st_filesystem_i->fs_name[sizeof(st_filesystem_i->fs_name) - 1] = '\0';
+			strncpy(st_filesystem_i->mountp, mountp, sizeof(st_filesystem_i->mountp));
+			st_filesystem_i->mountp[sizeof(st_filesystem_i->mountp) - 1] = '\0';
 		}
 	}
 
@@ -2665,8 +2686,8 @@ __nr_t read_fchost(struct stats_fchost *st_fc, __nr_t nr_alloc)
 			st_fc_i->f_txframes = tx_frames;
 			st_fc_i->f_rxwords  = rx_words;
 			st_fc_i->f_txwords  = tx_words;
-			memcpy(st_fc_i->fchost_name, drd->d_name, MAX_FCH_LEN);
-			st_fc_i->fchost_name[MAX_FCH_LEN - 1] = '\0';
+			memcpy(st_fc_i->fchost_name, drd->d_name, sizeof(st_fc_i->fchost_name));
+			st_fc_i->fchost_name[sizeof(st_fc_i->fchost_name) - 1] = '\0';
 		}
 	}
 
@@ -2724,5 +2745,165 @@ int read_softnet(struct stats_softnet *st_softnet, __nr_t nr_alloc,
 	return 1;
 }
 
+/*
+ ***************************************************************************
+ * Read pressure-stall information from a file located in /proc/pressure
+ * directory.
+ *
+ * IN:
+ * @st_psi	Structure where stats will be saved.
+ * @filename	File located in /proc/pressure directory to read.
+ * @token	"some" or "full". Indicate which line shall be read in file.
+ *
+ * OUT:
+ * @st_psi	Structure with statistics.
+ *
+ * RETURNS:
+ * 1 on success, 0 otherwise.
+ ***************************************************************************
+ */
+int read_psi_stub(struct stats_psi *st_psi, char *filename, char *token)
+{
+	FILE *fp;
+	char line[8192];
+	unsigned long psi_tmp[3];
+	int rc = 0, len;
+
+	if ((fp = fopen(filename, "r")) == NULL)
+		return 0;
+
+	len = strlen(token);
+	while (fgets(line, sizeof(line), fp) != NULL) {
+
+		if (!strncmp(line, token, len)) {
+			/* Read stats */
+			rc = sscanf(line + len + 1, "avg10=%lu.%lu avg60=%lu.%lu avg300=%lu.%lu total=%llu",
+				    &psi_tmp[0], &st_psi->avg10,
+				    &psi_tmp[1], &st_psi->avg60,
+				    &psi_tmp[2], &st_psi->avg300,
+				    &st_psi->total);
+		}
+	}
+
+	fclose(fp);
+
+	if (rc < 7)
+		return 0;
+
+	st_psi->avg10  += psi_tmp[0] * 100;
+	st_psi->avg60  += psi_tmp[1] * 100;
+	st_psi->avg300 += psi_tmp[2] * 100;
+
+	return 1;
+}
+
+/*
+ ***************************************************************************
+ * Read pressure-stall CPU information.
+ *
+ * IN:
+ * @st_psi_cpu	Structure where stats will be saved.
+ *
+ * OUT:
+ * @st_psi_cpu	Structure with statistics.
+ *
+ * RETURNS:
+ * 1 on success, 0 otherwise.
+ ***************************************************************************
+ */
+__nr_t read_psicpu(struct stats_psi_cpu *st_psi_cpu)
+{
+	struct stats_psi st_psi;
+
+	/* Read CPU stats */
+	if (!read_psi_stub(&st_psi, PSI_CPU, "some"))
+		return 0;
+
+	st_psi_cpu->some_acpu_10   = st_psi.avg10;
+	st_psi_cpu->some_acpu_60   = st_psi.avg60;
+	st_psi_cpu->some_acpu_300  = st_psi.avg300;
+	st_psi_cpu->some_cpu_total = st_psi.total;
+
+	return 1;
+}
+
+/*
+ ***************************************************************************
+ * Read pressure-stall I/O information.
+ *
+ * IN:
+ * @st_psi_io	Structure where stats will be saved.
+ *
+ * OUT:
+ * @st_psi_io	Structure with statistics.
+ *
+ * RETURNS:
+ * 1 on success, 0 otherwise.
+ ***************************************************************************
+ */
+__nr_t read_psiio(struct stats_psi_io *st_psi_io)
+{
+	struct stats_psi st_psi;
+
+	/* Read I/O "some" stats */
+	if (!read_psi_stub(&st_psi, PSI_IO, "some"))
+		return 0;
+
+	st_psi_io->some_aio_10   = st_psi.avg10;
+	st_psi_io->some_aio_60   = st_psi.avg60;
+	st_psi_io->some_aio_300  = st_psi.avg300;
+	st_psi_io->some_io_total = st_psi.total;
+
+	/* Read I/O "full" stats */
+	if (!read_psi_stub(&st_psi, PSI_IO, "full"))
+		return 0;
+
+	st_psi_io->full_aio_10   = st_psi.avg10;
+	st_psi_io->full_aio_60   = st_psi.avg60;
+	st_psi_io->full_aio_300  = st_psi.avg300;
+	st_psi_io->full_io_total = st_psi.total;
+
+	return 1;
+}
+
+/*
+ ***************************************************************************
+ * Read pressure-stall memory information.
+ *
+ * IN:
+ * @st_psi_mem	Structure where stats will be saved.
+ *
+ * OUT:
+ * @st_psi_mem	Structure with statistics.
+ *
+ * RETURNS:
+ * 1 on success, 0 otherwise.
+ ***************************************************************************
+ */
+__nr_t read_psimem(struct stats_psi_mem *st_psi_mem)
+{
+	struct stats_psi st_psi;
+
+	/* Read memory "some" stats */
+	if (!read_psi_stub(&st_psi, PSI_MEM, "some"))
+		return 0;
+
+	st_psi_mem->some_amem_10   = st_psi.avg10;
+	st_psi_mem->some_amem_60   = st_psi.avg60;
+	st_psi_mem->some_amem_300  = st_psi.avg300;
+	st_psi_mem->some_mem_total = st_psi.total;
+
+	/* Read memory "full" stats */
+	if (!read_psi_stub(&st_psi, PSI_MEM, "full"))
+		return 0;
+
+	st_psi_mem->full_amem_10   = st_psi.avg10;
+	st_psi_mem->full_amem_60   = st_psi.avg60;
+	st_psi_mem->full_amem_300  = st_psi.avg300;
+	st_psi_mem->full_mem_total = st_psi.total;
+
+	return 1;
+}
+
 /*------------------ END: FUNCTIONS USED BY SADC ONLY ---------------------*/
 #endif /* SOURCE_SADC */
