@@ -1,6 +1,6 @@
 /*
  * sar and sadf common routines.
- * (C) 1999-2019 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -28,6 +28,7 @@
 #include <unistd.h>	/* For STDOUT_FILENO, among others */
 #include <dirent.h>
 #include <fcntl.h>
+#include <limits.h>
 #include <libgen.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -150,12 +151,12 @@ void guess_sa_name(char *sa_dir, struct tm *rectime, int *sa_name)
 	*sa_name = 0;
 
 	/* Look for saYYYYMMDD */
-	snprintf(filename, MAX_FILE_LEN,
+	snprintf(filename, sizeof(filename),
 		 "%s/sa%04d%02d%02d", sa_dir,
 		 rectime->tm_year + 1900,
 		 rectime->tm_mon + 1,
 		 rectime->tm_mday);
-	filename[MAX_FILE_LEN - 1] = '\0';
+	filename[sizeof(filename) - 1] = '\0';
 
 	if (stat(filename, &sb) < 0)
 		/* Cannot find or access saYYYYMMDD, so use saDD */
@@ -164,10 +165,10 @@ void guess_sa_name(char *sa_dir, struct tm *rectime, int *sa_name)
 	nsec = sb.st_mtim.tv_nsec;
 
 	/* Look for saDD */
-	snprintf(filename, MAX_FILE_LEN,
+	snprintf(filename, sizeof(filename),
 		 "%s/sa%02d", sa_dir,
 		 rectime->tm_mday);
-	filename[MAX_FILE_LEN - 1] = '\0';
+	filename[sizeof(filename) - 1] = '\0';
 
 	if (stat(filename, &sb) < 0) {
 		/* Cannot find or access saDD, so use saYYYYMMDD */
@@ -211,12 +212,12 @@ int set_default_file(char *datafile, int d_off, int sa_name)
 
 	/* Set directory where daily data files will be saved */
 	if (datafile[0]) {
-		strncpy(sa_dir, datafile, MAX_FILE_LEN);
+		strncpy(sa_dir, datafile, sizeof(sa_dir));
 	}
 	else {
-		strncpy(sa_dir, SA_DIR, MAX_FILE_LEN);
+		strncpy(sa_dir, SA_DIR, sizeof(sa_dir));
 	}
-	sa_dir[MAX_FILE_LEN - 1] = '\0';
+	sa_dir[sizeof(sa_dir) - 1] = '\0';
 
 	get_time(&rectime, d_off);
 	if (sa_name < 0) {
@@ -275,17 +276,13 @@ int set_default_file(char *datafile, int d_off, int sa_name)
  */
 int check_alt_sa_dir(char *datafile, int d_off, int sa_name)
 {
-	struct stat sb;
-
-	if (stat(datafile, &sb) == 0) {
-		if (S_ISDIR(sb.st_mode)) {
-			/*
-			 * This is a directory: So append
-			 * the default file name to it.
-			 */
-			set_default_file(datafile, d_off, sa_name);
-			return 1;
-		}
+	if (check_dir(datafile)) {
+		/*
+		 * This is a directory: So append
+		 * the default file name to it.
+		 */
+		set_default_file(datafile, d_off, sa_name);
+		return 1;
 	}
 
 	return 0;
@@ -528,82 +525,6 @@ void reallocate_all_buffers(struct activity *a, __nr_t nr_min)
 	a->nr_allocated = nr_realloc;
 }
 
-/*
-  ***************************************************************************
-  * Try to get device real name from sysfs tree.
-  *
-  * IN:
-  * @major	Major number of the device.
-  * @minor	Minor number of the device.
-  *
-  * RETURNS:
-  * The name of the device, which may be the real name (as it appears in /dev)
-  * or NULL.
-  ***************************************************************************
-  */
-char *get_devname_from_sysfs(unsigned int major, unsigned int minor)
-{
-	static char link[256], target[PATH_MAX];
-	char *devname;
-	ssize_t r;
-
-	snprintf(link, 256, "%s/%u:%u", SYSFS_DEV_BLOCK, major, minor);
-
-	/* Get full path to device knowing its major and minor numbers */
-	r = readlink(link, target, PATH_MAX);
-	if (r <= 0 || r >= PATH_MAX) {
-		return (NULL);
-	}
-
-	target[r] = '\0';
-
-	/* Get device name */
-	devname = basename(target);
-	if (!devname || strnlen(devname, FILENAME_MAX) == 0) {
-		return (NULL);
-	}
-
-	return (devname);
-}
-
-/*
- ***************************************************************************
- * Get device real name if possible.
- * Warning: This routine may return a bad name on 2.4 kernels where
- * disk activities are read from /proc/stat.
- *
- * IN:
- * @major	Major number of the device.
- * @minor	Minor number of the device.
- * @pretty	TRUE if the real name of the device (as it appears in /dev)
- * 		should be returned.
- *
- * RETURNS:
- * The name of the device, which may be the real name (as it appears in /dev)
- * or a string with the following format devM-n.
- ***************************************************************************
- */
-char *get_devname(unsigned int major, unsigned int minor, int pretty)
-{
-	static char buf[32];
-	char *name;
-
-	snprintf(buf, 32, "dev%u-%u", major, minor);
-
-	if (!pretty)
-		return (buf);
-
-	name = get_devname_from_sysfs(major, minor);
-	if (name != NULL)
-		return (name);
-
-	name = ioc_name(major, minor);
-	if ((name != NULL) && strcmp(name, K_NODEV))
-		return (name);
-
-	return (buf);
-}
-
 /*
  ***************************************************************************
  * Check if we are close enough to desired interval.
@@ -617,7 +538,7 @@ char *get_devname(unsigned int major, unsigned int minor, int pretty)
  * @interval	Interval of time.
  *
  * RETURNS:
- * 1 if we are actually close enough to desired interval, 0 otherwise.
+ * TRUE if we are actually close enough to desired interval, FALSE otherwise.
  ***************************************************************************
 */
 int next_slice(unsigned long long uptime_ref, unsigned long long uptime,
@@ -642,6 +563,10 @@ int next_slice(unsigned long long uptime_ref, unsigned long long uptime,
 
 	last_uptime = uptime;
 
+	if (interval == 1)
+		/* Smallest time interval: Always close enough to desired interval */
+		return TRUE;
+
 	/*
 	 * A few notes about the "algorithm" used here to display selected entries
 	 * from the system activity file (option -f with -i flag):
@@ -1063,7 +988,7 @@ int check_net_edev_reg(struct activity *a, int curr, int ref, int pos)
 
 /*
  ***************************************************************************
- * Disks may be registered dynamically (true in /proc/stat file).
+ * Disks may be registered dynamically (true in /proc/diskstats file).
  * This is what we try to guess here.
  *
  * IN:
@@ -1921,9 +1846,6 @@ int sa_open_read_magic(int *fd, char *dfile, struct file_magic *file_magic,
  * @dfile	Name of system activity data file.
  * @act		Array of activities.
  * @flags	Flags for common options and system state.
- * @ignore	Set to 1 if a true sysstat activity file but with a bad
- *		format should not yield an error message. Used with
- *		sadf -H (sadf -c doesn't call check_file_actlst() function).
  *
  * OUT:
  * @ifd		System activity data file descriptor.
@@ -1941,16 +1863,17 @@ int sa_open_read_magic(int *fd, char *dfile, struct file_magic *file_magic,
 void check_file_actlst(int *ifd, char *dfile, struct activity *act[], uint64_t flags,
 		       struct file_magic *file_magic, struct file_header *file_hdr,
 		       struct file_activity **file_actlst, unsigned int id_seq[],
-		       int ignore, int *endian_mismatch, int *arch_64)
+		       int *endian_mismatch, int *arch_64)
 {
-	int i, j, k, p;
+	int i, j, k, p, skip;
 	struct file_activity *fal;
 	void *buffer = NULL;
 	size_t bh_size = FILE_HEADER_SIZE;
 	size_t ba_size = FILE_ACTIVITY_SIZE;
 
 	/* Open sa data file and read its magic structure */
-	if (sa_open_read_magic(ifd, dfile, file_magic, ignore, endian_mismatch, TRUE) < 0)
+	if (sa_open_read_magic(ifd, dfile, file_magic,
+			       DISPLAY_HDR_ONLY(flags), endian_mismatch, TRUE) < 0)
 		/*
 		 * Not current sysstat's format.
 		 * Return now so that sadf -H can display at least
@@ -2076,17 +1999,19 @@ void check_file_actlst(int *ifd, char *dfile, struct activity *act[], uint64_t f
 			/* Unknown activity */
 			continue;
 
-		if (act[p]->magic != fal->magic) {
+		skip = FALSE;
+		if (fal->magic != act[p]->magic) {
 			/* Bad magical number */
-			if (ignore) {
+			if (DISPLAY_HDR_ONLY(flags)) {
 				/*
 				 * This is how sadf -H knows that this
 				 * activity has an unknown format.
 				 */
 				act[p]->magic = ACTIVITY_MAGIC_UNKNOWN;
 			}
-			else
-				continue;
+			else {
+				skip = TRUE;
+			}
 		}
 
 		/* Check max value for known activities */
@@ -2111,7 +2036,16 @@ void check_file_actlst(int *ifd, char *dfile, struct activity *act[], uint64_t f
 		     ||
 		     ((fal->types_nr[0] <= act[p]->gtypes_nr[0]) &&
 		     (fal->types_nr[1] <= act[p]->gtypes_nr[1]) &&
-		     (fal->types_nr[2] <= act[p]->gtypes_nr[2]))) && !ignore) {
+		     (fal->types_nr[2] <= act[p]->gtypes_nr[2]))) &&
+		     (act[p]->magic != ACTIVITY_MAGIC_UNKNOWN) && !DISPLAY_HDR_ONLY(flags)) {
+			/*
+			 * This may not be an error (that's actually why we may have changed
+			 * the magic number for this activity above).
+			 * So, if the activity magic number has changed (e.g.: ACTIVITY_MAGIC_UNKNOWN)
+			 * and we want to display only the header, then ignore the error.
+			 * If we want to also display the stats then we must stop here because
+			 * we won't know how to map the contents of the stats structure.
+			 */
 #ifdef DEBUG
 			fprintf(stderr, "%s: id=%d file=%d,%d,%d activity=%d,%d,%d\n",
 				__FUNCTION__, fal->id, fal->types_nr[0], fal->types_nr[1], fal->types_nr[2],
@@ -2128,6 +2062,13 @@ void check_file_actlst(int *ifd, char *dfile, struct activity *act[], uint64_t f
 			goto format_error;
 		}
 
+		if (skip)
+			/*
+			 * This is an unknown activity and we want stats about it:
+			 * This is not possible so skip it.
+			 */
+			continue;
+
 		for (k = 0; k < 3; k++) {
 			act[p]->ftypes_nr[k] = fal->types_nr[k];
 		}
@@ -2152,6 +2093,7 @@ void check_file_actlst(int *ifd, char *dfile, struct activity *act[], uint64_t f
 	}
 
 	free(buffer);
+	buffer = NULL;
 
 	/* Check that at least one activity selected by the user is available in file */
 	for (i = 0; i < NR_ACT; i++) {
@@ -2237,7 +2179,7 @@ int parse_sar_opt(char *argv[], int *opt, struct activity *act[],
 			 * Force '-r ALL -u ALL -F'.
 			 * Setting -F is compulsory because corresponding activity
 			 * has AO_MULTIPLE_OUTPUTS flag set.
-			 * -P ALL / -I ALL will be set only corresponding option has
+			 * -P ALL / -I ALL will be set only if corresponding option has
 			 * not been exlicitly entered on the command line.
 			 */
 			p = get_activity_position(act, A_MEMORY, EXIT_IF_NOT_FOUND);
@@ -2284,11 +2226,8 @@ int parse_sar_opt(char *argv[], int *opt, struct activity *act[],
 			break;
 
 		case 'h':
-			/*
-			 * Make output easier to read by a human.
-			 * Option -h implies --human and -p (pretty-print).
-			 */
-			*flags |= S_F_HUMAN_READ + S_F_UNIT + S_F_DEV_PRETTY;
+			/* Option -h is equivalent to --pretty --human */
+			*flags |= S_F_PRETTY + S_F_UNIT;
 			break;
 
 		case 'j':
@@ -2297,33 +2236,31 @@ int parse_sar_opt(char *argv[], int *opt, struct activity *act[],
 			}
 			(*opt)++;
 			if (!strcmp(argv[*opt], K_SID)) {
-				*flags |= S_F_DEV_SID + S_F_DEV_PRETTY;
+				*flags |= S_F_DEV_SID + S_F_PRETTY;
 				return 0;
 			}
 
-			if (strnlen(argv[*opt], MAX_FILE_LEN) >= MAX_FILE_LEN - 1)
+			if (strnlen(argv[*opt], sizeof(persistent_name_type)) >= sizeof(persistent_name_type) - 1)
 				return 1;
 
-			strncpy(persistent_name_type, argv[*opt], MAX_FILE_LEN - 1);
-			persistent_name_type[MAX_FILE_LEN - 1] = '\0';
+			strncpy(persistent_name_type, argv[*opt], sizeof(persistent_name_type) - 1);
+			persistent_name_type[sizeof(persistent_name_type) - 1] = '\0';
 			strtolower(persistent_name_type);
 			if (!get_persistent_type_dir(persistent_name_type)) {
 				fprintf(stderr, _("Invalid type of persistent device name\n"));
 				return 2;
 			}
-			/*
-			 * If persistent device name doesn't exist for device, use
-			 * its pretty name.
-			 */
-			*flags |= S_F_PERSIST_NAME + S_F_DEV_PRETTY;
+			/* Pretty print report (option -j implies option -p) */
+			*flags |= S_F_PERSIST_NAME + S_F_PRETTY;
 			return 0;
 			break;
 
 		case 'p':
-			*flags |= S_F_DEV_PRETTY;
+			*flags |= S_F_PRETTY;
 			break;
 
 		case 'q':
+			/* Option -q grouped with other ones */
 			SELECT_ACTIVITY(A_QUEUE);
 			break;
 
@@ -2565,6 +2502,57 @@ int parse_sar_n_opt(char *argv[], int *opt, struct activity *act[])
 	return 0;
 }
 
+/*
+ ***************************************************************************
+ * Parse sar "-q" option.
+ *
+ * IN:
+ * @argv	Arguments list.
+ * @opt		Index in list of arguments.
+ *
+ * OUT:
+ * @act		Array of selected activities.
+ *
+ * RETURNS:
+ * 0 on success, 1 otherwise.
+ ***************************************************************************
+ */
+int parse_sar_q_opt(char *argv[], int *opt, struct activity *act[])
+{
+	char *t;
+
+	for (t = strtok(argv[*opt], ","); t; t = strtok(NULL, ",")) {
+		if (!strcmp(t, K_LOAD)) {
+			SELECT_ACTIVITY(A_QUEUE);
+		}
+		else if (!strcmp(t, K_PSI_CPU)) {
+			SELECT_ACTIVITY(A_PSI_CPU);
+		}
+		else if (!strcmp(t, K_PSI_IO)) {
+			SELECT_ACTIVITY(A_PSI_IO);
+		}
+		else if (!strcmp(t, K_PSI_MEM)) {
+			SELECT_ACTIVITY(A_PSI_MEM);
+		}
+		else if (!strcmp(t, K_PSI)) {
+			SELECT_ACTIVITY(A_PSI_CPU);
+			SELECT_ACTIVITY(A_PSI_IO);
+			SELECT_ACTIVITY(A_PSI_MEM);
+		}
+		else if (!strcmp(t, K_ALL)) {
+			SELECT_ACTIVITY(A_QUEUE);
+			SELECT_ACTIVITY(A_PSI_CPU);
+			SELECT_ACTIVITY(A_PSI_IO);
+			SELECT_ACTIVITY(A_PSI_MEM);
+		}
+		else
+			return 1;
+	}
+
+	(*opt)++;
+	return 0;
+}
+
 /*
  ***************************************************************************
  * Parse sar "-I" option.
@@ -3319,58 +3307,4 @@ void get_global_soft_statistics(struct activity *a, int prev, int curr,
 		ssnp_all->flow_limit += ssnp->flow_limit;
 	}
 }
-
-/*
- ***************************************************************************
- * Get device name (whether pretty-printed, persistent or not).
- *
- * IN:
- * @major	Major number of the device.
- * @minor	Minor number of the device.
- * @wwn		WWN identifier of the device (0 if unknown).
- * @part_nr	Partition number (0 if unknown).
- * @flags	Flags for common options and system state.
- *
- * RETURNS:
- * The name of the device.
- ***************************************************************************
- */
-char *get_sa_devname(unsigned int major, unsigned int minor, unsigned long long wwn[],
-		     unsigned int part_nr, uint64_t flags)
-{
-	char *dev_name = NULL, *persist_dev_name = NULL;
-	static char sid[64];
-	char xsid[32] = "", pn[16] = "";
-
-	if (DISPLAY_PERSIST_NAME_S(flags)) {
-		persist_dev_name = get_persistent_name_from_pretty(get_devname(major, minor, TRUE));
-	}
-
-	if (persist_dev_name) {
-		dev_name = persist_dev_name;
-	}
-	else {
-		if ((USE_STABLE_ID(flags)) && (wwn[0] != 0)) {
-			if (wwn[1] != 0) {
-				sprintf(xsid, "%016llx", wwn[1]);
-			}
-			if (part_nr) {
-				sprintf(pn, "-%d", part_nr);
-			}
-			snprintf(sid, sizeof(sid), "%#016llx%s%s", wwn[0], xsid, pn);
-			dev_name = sid;
-		}
-		else if ((USE_PRETTY_OPTION(flags)) && (major == dm_major)) {
-			dev_name = transform_devmapname(major, minor);
-		}
-
-		if (!dev_name) {
-			dev_name = get_devname(major, minor,
-					       USE_PRETTY_OPTION(flags));
-		}
-	}
-
-	return dev_name;
-}
-
 #endif /* SOURCE_SADC undefined */
