@@ -63,7 +63,6 @@ int arch_64 = FALSE;
 int dplaces_nr = -1;
 
 uint64_t flags = 0;
-unsigned int dm_major;	/* Device-mapper major number */
 
 char timestamp[2][TIMESTAMP_LEN];
 extern unsigned int rec_types_nr[];
@@ -128,7 +127,7 @@ void usage(char *progname)
 			  "[ -m { <keyword> [,...] | ALL } ] [ -n { <keyword> [,...] | ALL } ]\n"
 			  "[ -q [ <keyword> [,...] | ALL ] ]\n"
 			  "[ --dev=<dev_list> ] [ --fs=<fs_list> ] [ --iface=<iface_list> ]\n"
-			  "[ --dec={ 0 | 1 | 2 } ] [ --help ] [ --human ] [ --sadc ]\n"
+			  "[ --dec={ 0 | 1 | 2 } ] [ --help ] [ --human ] [ --pretty ] [ --sadc ]\n"
 			  "[ -j { SID | ID | LABEL | PATH | UUID | ... } ]\n"
 			  "[ -f [ <filename> ] | -o [ <filename> ] | -[0-9]+ ]\n"
 			  "[ -i <interval> ] [ -s [ <hh:mm[:ss]> ] ] [ -e [ <hh:mm[:ss]> ] ]\n"));
@@ -1341,6 +1340,12 @@ int main(int argc, char **argv)
 			opt++;
 		}
 
+		else if (!strcmp(argv[opt], "--pretty")) {
+			/* Display an easy-to-read report */
+			flags |= S_F_PRETTY;
+			opt++;
+		}
+
 		else if (!strncmp(argv[opt], "--dec=", 6) && (strlen(argv[opt]) == 7)) {
 			/* Get number of decimal places */
 			dplaces_nr = atoi(argv[opt] + 6);
@@ -1563,10 +1568,6 @@ int main(int argc, char **argv)
 		usage(argv[0]);
 	}
 
-	if (USE_PRETTY_OPTION(flags)) {
-		dm_major = get_devmap_major();
-	}
-
 	if (!count) {
 		/*
 		 * count parameter not set: Display all the contents of the file
